function [J_sigma, J_sigma_d] = comp_actuation_task_jacobians(snake, B, q, q_dot)
% This function computes the derivative of the task Jacobian.
%   Detailed explanation goes here

%%
n_q = snake.n - 1;

BBT = B*B';
sigma = det(BBT);
%B_plus = B'/BBT;
B_plus = pinv(B);
J_sigma = zeros(1, 6 + n_q);
% Note that the hessian is not the entire Jacobian, but rather the nonzero
% elements on the right.
hessian = zeros(n_q, n_q);

dBB_plus = zeros(6 + n_q, 6 + n_q, n_q);

dSigma = zeros(1, n_q);

% Compute dB and ddB from precomputed functions and q.
% Note that these functions are hard-coded with neccessary dimension in q.
d_B = dB(q(1), q(2), q(3), q(4), q(5), q(6));
dd_B = ddB(q(1), q(2), q(3), q(4), q(5), q(6));

 for i = 1:n_q
     dBB_plus(:, :, i) = d_B(:, :, i)*B_plus;
     dSigma(i) = 2*sigma*trace(dBB_plus(:,:,i));
     J_sigma(i+6) = dSigma(i);
 end
 
% Is the non-zero area of the Jacobian symmetric due to cross-partials?
% Requirement is for the elements to be continuously differentiable?
for j = 1:n_q
   for i = 1:n_q
       hessian(j, i) = ...
           2*dSigma(j)*trace(dBB_plus(i)) ...
           + 2*sigma*trace(dd_B(:,:,i,j)*B_plus ...
           + d_B(:,:,i)*(d_B(:,:,j)' - B_plus*(d_B(:,:,j)*B' + B*(d_B(:,:,j))'))/BBT);
   end
end

J_sigma_d = q_dot'*hessian';
J_sigma_d = [zeros(1, 6) J_sigma_d];
end

