function x_dot = dyn_accel_nosmooth(x)
snake = evalin('base', 'snake');
pp = evalin('base','pp');

%Get states
% snake     =  x(end)
pos       =  x(1:3);
quat      =  x(4:7);
theta     =  x(8:15);
zeta      =  x(16:29);
t         =  x(30);
u         =  x(31:45);
att_d     =  x(46:48);
sigma_1d  =  x(49:51);


persistent init_flag;

if isempty(init_flag)
    init_flag = 0;
end

%Compute rotation matrices and jacobians
[R, J]    =  comp_rot_jac(quat,theta,snake);

J_dot    = comp_jac_dot(theta, snake, J, zeta(7:end));


%Compute mass matrix
M         =  comp_mass(J,snake);

%Compute relative velocity to take into account the effect of current
velc_h    =  R(:,:,1)'*pp.velc;
nu        =  zeta(1:6);
zeta(1:3) =  zeta(1:3)-velc_h;

%Compute coriolis, drag, and hydrostatic terms with the relative velocity
[c, d, g] =  comp_cdg(J,R,theta,snake,pp,zeta);
%Compute the thrust-configuration matrix
B         =  comp_TCM(J,snake);

%testing with a swimming gait, add your own controller here:
    % Homogenous transformation matrix between body-fixed base frame and
    % end-effector frame
    H_be = snake.joint(1).A(theta(1))*snake.joint(2).A(theta(2))*snake.joint(3).A(theta(3))*snake.joint(4).A(theta(4))*snake.joint(5).A(theta(5))*snake.joint(6).A(theta(6))*snake.joint(7).A(theta(7))*snake.joint(8).A(theta(8));
%     H_be_d = snake.joint(1).A(theta_d(1))*snake.joint(2).A(theta_d(2))*snake.joint(3).A(theta_d(3))*snake.joint(4).A(theta_d(4))*snake.joint(5).A(theta_d(5))*snake.joint(6).A(theta_d(6))*snake.joint(7).A(theta_d(7))*snake.joint(8).A(theta_d(8));
    % Position of end-effector in body-fixed base frame.
    pos_endeff = H_be*[snake.link(9).length;0;0;1];
    p_endeff = pos + R(:,:,1)*pos_endeff(1:3);
    A_e = [eye(3) [snake.link(9).length;0;0];
            zeros(1,3) 1];
    
    
    eulang_endeff = rotm2eul(R(:,:,9), 'XYZ')';
    quat_endeff = rotm2quat(R(:,:,9))';
    T_0b = TransAngQ(quat_endeff);
    
    V_e = Ad_inv(A_e)*J(:,:,9)*zeta;
    q_dot = T_0b*V_e(4:6);

     
     
%     J_1 = [R_d(:,:,9) zeros(3,3); zeros(3,3) T_0b(2:4,:)]*J_d(:,:,9);
    J_1 = [R(:,:,9) zeros(3,3); zeros(3,3) T_0b(2:4,:)]*Ad_inv(A_e)*J(:,:,9);
    T_0b_dot = 1/2.*[0 -1 0 0 0 0 -1 0 0 0 0 -1;
                1 0 0 0 0 0 0 -1 0 0 1 0;
                0 0 0 1 1 0 0 0 0 -1 0 0;
                0 0 -1 0 0 1 0 0 1 0 0 0]*kron(eye(3),q_dot);
    J_1_dot = [R(:,:,9)*Smtrx(V_e(4:6)) zeros(3,3); zeros(3,3) T_0b_dot(2:4,:)]*Ad_inv(A_e)*J(:,:,9) + [R(:,:,9) zeros(3,3); zeros(3,3) T_0b(2:4,:)]*Ad_inv(A_e)*J_dot(:,:,9);
  
    J_2 = [R(:,:,1) zeros(3,3+snake.n-1)];      % Base Jacobian
    J_2_dot = [R(:,:,1)*Smtrx(zeta(4:6)) zeros(3,3+snake.n-1)];
    
    theta_max = 60*pi/180;
    sigma_b_max = theta_max.* ones(8,1);
    sigma_b_min = -sigma_b_max;

    pos_obs = [2.5 0.5 -10]';
    
    sigma_a1 = sqrt((pos_obs-p_endeff)'*(pos_obs-p_endeff));
    J_a_1 = -(pos_obs-p_endeff)'/(sigma_a1)*J_1(1:3,:);
   
       
    sigma_2d = [0 0 -10]';
    
    
    B_joint = [zeros(6,8);eye(8)];
    B_full = [B B_joint];
    act_index = det(B_full*B_full');

% 
% 
%     N_2 = eye(14) - J_1'/((J_1/M)*J_1')*(J_1/M);
% %     J_2_bar = M\(J_2'*M_2);
% %     N_2 = N_1_bar*(eye(14)-J_2'*J_2_bar'*N_1_bar);
%     M_2 = comp_ctrlable_dir(J_2,N_2,M);
%     J_2_bar = M\(J_2'*M_2);
%     N_3 = N_2*(eye(14)-J_2'*J_2_bar'*N_2);
% 
% 
%     A_1 = [J_1/M*J_1' zeros(6,3) zeros(6,size(J_3,1))];
%     A_2 = [J_2/M*J_1' J_2/M*N_2*J_2' zeros(3,size(J_3,1))];
%     A_3 = [J_3/M*J_1' J_3/M*N_2*J_2' J_3/M*N_3*J_3'];
%     A = [A_1; A_2;A_3];
%     
%     p1 = 5;
%     p2 = 5;
%     p3 = 0.2;
%     P12 = diag([p1 p2 p3]);
%     H = 2*[P12 zeros(3,9+size(J_3,1));
%         zeros(9+m_3,3) A'*A];
%     
%     Lf1 = [J_1_dot*zeta - J_1/M*(c+d+g)];
%     Lf2 =     [J_2_dot*zeta - J_2/M*(c+d+g)];
%     Lf3 = [-J_3/M*(c+d+g)];
%     
%     
%     Lf = [Lf1;Lf2;Lf3];
%     
%     
%     F1 = [zeros(6,6) eye(6);zeros(6,12)];
%     F2 = [zeros(3,3) eye(3); zeros(3,6)];
% %     F3 = [zeros(m_3,m_3) eye(m_3); zeros(m_3, 2*m_3)];
%     F3 = zeros(m_3,m_3);
%     G1 = [zeros(6,6); eye(6)];
%     G2 = [zeros(3,3); eye(3)];
% %     G3 = [zeros(m_3,m_3);eye(m_3)];
%     G3 = eye(m_3);
% 
%     Q1 = eye(12);
%     Q2 = eye(6);
%     Q3 = eye(m_3);
%     
%     [P1, ~, ~] = care(F1,G1,Q1);
%     [P2, ~, ~] = care(F2,G2,Q2);
%     [P3, ~, ~] = care(F3,G3,Q3);
%     
% 
%     
%     gamma_1 = .5;
%     gamma_2 = 0.1;
%     gamma_3 = 1;
%     
%     eta_1 = [sigma_1_tilde; sigma_1_tilde_dot];
%     eta_2 = [sigma_2_tilde; sigma_2_tilde_dot];
%     eta_3 = [sigma_3_tilde_dot];
%     
%     
% 
%     
%     psi_11 = (2*eta_1'*P1*G1)';
%     psi_12 = (2*eta_2'*P2*G2)';
%     psi_13 = (2*eta_3'*P3*G3)';
%     
%     psi_01 = eta_1'*(F1'*P1+P1*F1)*eta_1 + gamma_1*eta_1'*P1*eta_1;
%     psi_02 = eta_2'*(F2'*P2+P2*F2)*eta_2 + gamma_2*eta_2'*P2*eta_2;
%     psi_03 = eta_3'*(F3'*P3+P3*F3)*eta_3 + gamma_3*eta_3'*P3*eta_3;
%     
%     Aineq = [-1 0 0 psi_11'*A_1;
%         0 -1 0 psi_12'*A_2;
%         0 0 -1 psi_13'*A_3];
%     b = [-psi_01 - psi_11'*Lf1;
%          -psi_02 - psi_12'*Lf2;
%          -psi_03 - psi_13'*Lf3];
%     
%     f = [0 0 0 2*Lf'*A];
% 
% %     lb = zeros(size(H,1),1);
% %     ub = zeros(size(H,1),1);
% %    
% %     lb(4:end) = -60*ones(17,1);
% %     ub(4:end) = 60*ones(17,1);
% %     
% %     z0 = zeros(size(H,1),1);
% %     u = quadprog(H,f,Aineq,b,[],[],lb,ub,z0);
%     u = quadprog(H,f,Aineq,b);
%     u1 = u(4:9);
%     u2 = u(10:12);
%     u3 = u(13:end);
% 
%      
%     tau = J_1'*u1 + N_2*J_2'*u2 + N_3*J_3'*u3;
%     W = diag([0.1 0.1 0.1 0.1 0.1 0.1 0.1 1 1 1 1 1 1 1 1]);
%     u = W\B_full'/((B_full/W)*B_full'+0.00*eye(size(B_full,1)))*tau;
    tau = zeros(14,1);
    thrust = u(1:7);
    joint_torque = u(8:end);



% disp(['Elapsed time: ' num2str(t)])

t_prev = t;
%Compute the relative acceleration
zeta_dot      =  M\(B*thrust+[zeros(6,1);joint_torque]-c-d-g);
% zeta_dot      =  M\(tau-c-d-g);
%Remove the effect of current
zeta_dot(1:3) = zeta_dot(1:3)-cross(nu(4:6),velc_h);
%Compute the time derivatives of position and orientation
pos_dot   =  R(:,:,1)*nu(1:3);
quat_dot  =  TransAngQ(quat)*nu(4:6)+1*(1-quat'*quat)*quat; 

% Integral state in feedback linearizing controller
% zeta_d_dot = zeta-zeta_d;
%Assemble derivative of the state vector
x_dot = [pos_dot ; quat_dot ; zeta(7:snake.n+5) ; zeta_dot;pos; zeta; thrust; joint_torque; p_endeff; eulang_endeff; sigma_1d;att_d; sigma_2d; theta; 1;tau; sigma_a1;J_a_1*zeta;act_index;quat_endeff];

% nin  = nargin;
% if nin == 5 && flag ==1
%     x_dot = [pos; zeta; thrust; joint_torque; zeta_d; p_endeff; eulang_endeff; sigma_1d;att_d; sigma_2d; theta; theta_d; mode];
% end

end
