function x_dot = dyn_accel_smooth(x)
snake = evalin('base', 'snake');
pp = evalin('base','pp');
K_1 = evalin('base', 'K_1');
K_2 = evalin('base', 'K_2');
K_3 = evalin('base', 'K_3');
K_4 = evalin('base', 'K_4');
K_att_dot = evalin('base', 'K_att_dot');
K_att = evalin('base', 'K_att');
%Get states
% snake     =  x(end)
pos       =  x(1:3);
quat      =  x(4:7);
theta     =  x(8:15);
zeta      =  x(16:29);
t         =  x(30);

persistent init_flag;
persistent f1;
persistent f2;
persistent t_switch;
persistent mode;
persistent mode_prev;
persistent f2_prev;
persistent tasks;
persistent t_prev;

if isempty(init_flag)
    t_switch = -Inf;
    mode = 1;
    f2 = zeros(14,1);
    f2_prev = zeros(14,1);
    init_flag = 1;
    t_prev = 0;
end
% mode_prev = mode;
tasks_prev = tasks;


%Compute rotation matrices and jacobians
[R, J]    =  comp_rot_jac(quat,theta,snake);

J_dot    = comp_jac_dot(theta, snake, J, zeta(7:end));


%Compute mass matrix
M         =  comp_mass(J,snake);

%Compute relative velocity to take into account the effect of current
velc_h    =  R(:,:,1)'*pp.velc;
nu        =  zeta(1:6);
zeta(1:3) =  zeta(1:3)-velc_h;

%Compute coriolis, drag, and hydrostatic terms with the relative velocity
[c, d, g] =  comp_cdg(J,R,theta,snake,pp,zeta);
%Compute the thrust-configuration matrix
B         =  comp_TCM(J,snake);

%testing with a swimming gait, add your own controller here:
    % Homogenous transformation matrix between body-fixed base frame and
    % end-effector frame
    H_be = snake.joint(1).A(theta(1))*snake.joint(2).A(theta(2))*snake.joint(3).A(theta(3))*snake.joint(4).A(theta(4))*snake.joint(5).A(theta(5))*snake.joint(6).A(theta(6))*snake.joint(7).A(theta(7))*snake.joint(8).A(theta(8));
%     H_be_d = snake.joint(1).A(theta_d(1))*snake.joint(2).A(theta_d(2))*snake.joint(3).A(theta_d(3))*snake.joint(4).A(theta_d(4))*snake.joint(5).A(theta_d(5))*snake.joint(6).A(theta_d(6))*snake.joint(7).A(theta_d(7))*snake.joint(8).A(theta_d(8));
    % Position of end-effector in body-fixed base frame.
    pos_endeff = H_be*[snake.link(9).length;0;0;1];
%     pos_endeff_d = H_be_d*[snake.link(9).length;0;0;1];
    % Position of end-effecter in the inertial frame.
%     p_endeff = pos + R(:,:,1)*pos_endeff(1:3);
    p_endeff = pos + R(:,:,1)*pos_endeff(1:3);
%     p_endeff_d = pos_d + R_d(:,:,1)*pos_endeff_d(1:3);
    A_e = [eye(3) [snake.link(9).length;0;0];
            zeros(1,3) 1];
    
    
    eulang_endeff = rotm2eul(R(:,:,9), 'XYZ')';
    quat_endeff = rotm2quat(R(:,:,9))';
    T_0b = TransAngQ(quat_endeff);
    if t < 50
        psi_d = pi/2;
        phi_d = 0;
        theta_d = 5*pi/180;
        sigma_1d = [1;2;-10];
    elseif t < 100
        phi_d = 5*pi/180;
        psi_d = atan2(2.5,1.5);
        theta_d = 0*pi/180;
        sigma_1d = [1.5;2.5;-10];
    elseif t < 150
        phi_d = 0*pi/180;
        psi_d = pi/4;
        theta_d = -5*pi/180;
        sigma_1d = [2;2;-10];
    elseif t < 200
        phi_d = 0*pi/180;
        psi_d = atan2(2,2);
        theta_d = -0*pi/180;
        sigma_1d = [2;2;-10];
    elseif t < 250
        phi_d = -5*pi/180;
        psi_d = atan2(1,2);
        theta_d = 5*pi/180;
        sigma_1d = [2;1;-10];
    elseif t < 300
        phi_d = 0*pi/180;
        psi_d = 0;
        theta_d = -0*pi/180;
        sigma_1d = [2.5;0;-10];
    elseif t < 350
        phi_d = 3*pi/180;
        psi_d = atan2(0.7,3);
        theta_d = -3*pi/180;
        sigma_1d = [3;0.7;-10];
    else
        phi_d = 0*pi/180;
        psi_d = atan2(1,3.4);
        theta_d = 0*pi/180;
        sigma_1d = [3.4;1;-10];
    end
%     phi_d = -15*pi/180;
%     theta_d = 20*pi/180;
%     theta_d = 0;
    sigma_1d_dot = 0;
    omega_d = 0;
    att_d = [phi_d theta_d psi_d]';
    quat_d = eul2quat(att_d', 'XYZ')';
    eta_d = quat_d(1);
    epsilon_d = quat_d(2:4);
    eta = quat_endeff(1);
    epsilon = quat_endeff(2:4);
    % Quaternion error
    qe = [eta_d*eta+epsilon_d'*epsilon;
         eta*epsilon_d-eta_d*epsilon-Smtrx(epsilon_d)*epsilon];
    
    V_e = Ad_inv(A_e)*J(:,:,9)*zeta;
    q_dot = T_0b*V_e(4:6);

     
     
%     J_1 = [R_d(:,:,9) zeros(3,3); zeros(3,3) T_0b(2:4,:)]*J_d(:,:,9);
    J_1 = [R(:,:,9) zeros(3,3); zeros(3,3) T_0b(2:4,:)]*Ad_inv(A_e)*J(:,:,9);
    T_0b_dot = 1/2.*[0 -1 0 0 0 0 -1 0 0 0 0 -1;
                1 0 0 0 0 0 0 -1 0 0 1 0;
                0 0 0 1 1 0 0 0 0 -1 0 0;
                0 0 -1 0 0 1 0 0 1 0 0 0]*kron(eye(3),q_dot);
    J_1_dot = [R(:,:,9)*Smtrx(V_e(4:6)) zeros(3,3); zeros(3,3) T_0b_dot(2:4,:)]*Ad_inv(A_e)*J(:,:,9) + [R(:,:,9) zeros(3,3); zeros(3,3) T_0b(2:4,:)]*Ad_inv(A_e)*J_dot(:,:,9);
    
    J_2 = [R(:,:,1) zeros(3,3+snake.n-1)];      % Base Jacobian
    J_2_dot = [R(:,:,1)*Smtrx(zeta(4:6)) zeros(3,3+snake.n-1)];
    
    theta_max = 60*pi/180;
    theta_0 = 5*pi/180;
    sigma_b_max = (theta_max-theta_0).* ones(8,1);
    sigma_b_min = -sigma_b_max;
    %smoothing func
    a = 110;
    b = 0.05;

    pos_obs = [2.5 0.5 -10]';
    radius_obs = 0.3;
    epsilon = 0.05;
    
    sigma_a1 = sqrt((pos_obs-p_endeff)'*(pos_obs-p_endeff));
    J_a_1 = -(pos_obs-p_endeff)'/(sigma_a1)*J_1(1:3,:);
%     J_a1_dot = (p_endeff'*V_e(1:3)*sigma_a1^2+(pos_obstacle-p_endeff)'*ones(3,1)*V_e(1:3)'*(pos_obstacle - p_endeff))./sigma_a1^3*J_a_1 -(pos_obstacle-p_endeff)'/(sigma_a1)*([R(:,:,9)*Smtrx(V_e(4:6)) zeros(3,3)]*J(:,:,9)+[R(:,:,9) zeros(3,3)]*J_dot(:,:,9));
%     J_a1_dot = (ones(3,1)'*V_e(1:3)*sigma_a1^2-(pos_obstacle-p_endeff)'*ones(3,1)*V_e(1:3)'*(pos_obstacle - p_endeff))./sigma_a1^3*J_a_1 -(pos_obstacle-p_endeff)'/(sigma_a1)*([R(:,:,9)*Smtrx(V_e(4:6)) zeros(3,3)]*J(:,:,9)+[R(:,:,9) zeros(3,3)]*J_dot(:,:,9));
%     J_a1_dot = -(pos_obstacle-p_endeff)'/(sigma_a1)*([R(:,:,9)*Smtrx(V_e(4:6)) zeros(3,3)]*J(:,:,9)+[R(:,:,9) zeros(3,3)]*J_dot(:,:,9));
%     e = ones(3,1);
%     J_a1_dot = (R(:,:,9)*x    |V_e(1:3))'*(e*sigma_a1^2-(pos_obstacle-p_endeff)*e'*(pos_obstacle-p_endeff))./sigma_a1^3*J_a_1-(pos_obstacle-p_endeff)'/(sigma_a1)*([R(:,:,9)*Smtrx(V_e(4:6)) zeros(3,3)]*J(:,:,9)+[R(:,:,9) zeros(3,3)]*J_dot(:,:,9));
%     J_a1_dot = (sigma_a1^2-(pos_obstacle-p_endeff)'*(pos_obstacle-p_endeff))./sigma_a1^3*V_e(1:3)'*[R(:,:,9) zeros(3,3)]*J(:,:,9)-(pos_obstacle-p_endeff)'/(sigma_a1)*([R(:,:,9)*Smtrx(V_e(4:6)) zeros(3,3)]*J(:,:,9)+[R(:,:,9) zeros(3,3)]*J_dot(:,:,9));
%     jac_sigmadot_p = 1/sigma_a1^3.*[(pos_obs(2)-p_endeff(2))^2+(pos_obs(3)-p_endeff(3))^2 -(pos_obs(1) - p_endeff(1))*(pos_obs(2)-p_endeff(2)) -(pos_obs(1)-p_endeff(1))*(pos_obs(3)-p_endeff(3));
%                                    -(pos_obs(2)-p_endeff(2))*(pos_obs(1)-p_endeff(1)) (pos_obs(1)-p_endeff(1))^2+(pos_obs(3)-p_endeff(3))^2 -(pos_obs(2) - p_endeff(2))*(pos_obs(3)-p_endeff(3));
%                                    -(pos_obs(3) - p_endeff(3))*(pos_obs(1)-p_endeff(1)) -(pos_obs(3)-p_endeff(3))*(pos_obs(2)-p_endeff(2)) (pos_obs(1)-p_endeff(1))^2+(pos_obs(2)-p_endeff(2))^2];
    jac_sigmadot_p = eye(3)/sigma_a1 - (pos_obs-p_endeff)*(pos_obs-p_endeff)'/sigma_a1^3;
    J_a1_dot = (R(:,:,9)*V_e(1:3))'*jac_sigmadot_p*J_1(1:3,:)-(pos_obs-p_endeff)'/(sigma_a1)*J_1_dot(1:3,:);

    J_b = [0 0 0 0 0 0 1 0 0 0 0 0 0 0;
           0 0 0 0 0 0 0 1 0 0 0 0 0 0;
           0 0 0 0 0 0 0 0 1 0 0 0 0 0;
           0 0 0 0 0 0 0 0 0 1 0 0 0 0;
           0 0 0 0 0 0 0 0 0 0 1 0 0 0;
           0 0 0 0 0 0 0 0 0 0 0 1 0 0;
           0 0 0 0 0 0 0 0 0 0 0 0 1 0;
           0 0 0 0 0 0 0 0 0 0 0 0 0 1];
   
    

%     lambda_act = 0;
%     K_act = lambda_act^2;
    K_act = 0.1;
%     K_act_dot = 2*lambda_act;
    K_act_dot = 1;
    
    K_obs = 0.0;
    K_obs_dot = 1.2;
    
    Sigma_1 = [K_1*eye(3) zeros(3,3); zeros(3,3) K_att*eye(3)];
    sigma_1_tilde = [sigma_1d - p_endeff; qe(2:4)];
    sigma_1_tilde_dot = [0-R(:,:,9)*V_e(1:3); omega_d-R(:,:,9)*V_e(4:6)];
    Lambda_1_dot = [K_3*eye(3) zeros(3,3); zeros(3,3) K_att_dot*eye(3)];
    
%     sigma_2d = [0 -1 1.5]';
    sigma_2d = [0 0 -10]';
    
    Sigma_2 = K_2*eye(3);
    Lambda_2_dot = K_4*eye(3);
%     Sigma_2 = 0.1*eye(3);
    sigma_2_tilde = sigma_2d - pos;
    sigma_2_tilde_dot = - R(:,:,1)*zeta(1:3);

%     Lambda_3 = 0.6*eye(8);
    K_z_dot = .6;
    Lambda_3 = K_z_dot*eye(11);
    sigma_3_tilde_dot = -zeta(4:end);
    J_3 = [zeros(11,3) eye(11,11)];
%     J_3 = J_b;
    J_3_inv = J_3'/(J_3*J_3');
    
    B_joint = [zeros(6,8);eye(8)];
    B_full = [B B_joint];
    act_index = det(B_full*B_full');
    act_index_min = 0.1;
    [J_actuation, J_actuation_dot] = comp_actuation_task_jacobians(snake, B_full, theta, zeta(7:end));

    J_a = [J_a_1;J_b;J_actuation];
    J_a_dot = [J_a1_dot; zeros(8,14);J_actuation_dot];

    sigma_actuation_tilde = act_index_min-act_index;
    sigma_actu_tilde_dot = -J_actuation*zeta;
    
    J_1_inv = J_1'/(J_1*J_1');
    J_2_inv = J_2'/(J_2*J_2');
    N_1 = eye(14) - J_1_inv*J_1;
    J_12 = [J_1;J_2];
    J_12 = comp_lin_indep_cols(J_12')';
    J_12_inv = J_12'/(J_12*J_12');
    N_12 = eye(14)-J_12_inv*J_12;

    f1_accel = J_1_inv*(Sigma_1*sigma_1_tilde + Lambda_1_dot*sigma_1_tilde_dot - J_1_dot*zeta) + N_1*J_2_inv*(Sigma_2*sigma_2_tilde + Lambda_2_dot*sigma_2_tilde_dot - J_2_dot*zeta) + N_12*J_3_inv*Lambda_3*sigma_3_tilde_dot;
    % Extension to vector tasks
%     [Act,tasks] = calc_activation_matrix_mod(J_a*f1_accel + J_a_dot*f1, J_a*f1, [sigma_a1;theta], [radius_obs+0.1;sigma_b_min], [Inf;sigma_b_max]);
    zeta_d = zeta + (t-t_prev)*f1_accel;
    [Act,tasks] = calc_activation_matrix_mod(J_a*f1_accel, J_a*zeta_d, [sigma_a1;theta;act_index], [radius_obs+epsilon+0.05;sigma_b_min;act_index_min+0.03], [Inf;sigma_b_max;Inf]);
    if nnz(Act) == 0    
        if mode ~= 1
            t_switch = t;
            mode_prev = mode;
        end
        mode = 1;
        f1 = f1_accel;
        zeta_d_dot = (1-smoothing_function(t,t_switch,a,b)).*f2 + smoothing_function(t,t_switch,a,b).*f1;
    else
       J_a_act = Act*J_a;
       J_a_act_inv = J_a_act'/(J_a_act*J_a_act');
%        J_a_act_inv = pinv(J_a_act);
       N_a = (eye(14) - J_a_act_inv*J_a_act);
       J_a1 = [J_a_act;J_1];
%        J_a1_inv = pinv(J_a1);
       J_a1 = comp_lin_indep_cols(J_a1')';
       J_a1_inv = J_a1'/(J_a1*J_a1');
       N_a1 = (eye(14)-J_a1_inv*J_a1);
       Lambda_act = K_act_dot*eye(size(Act,1));
       Sigma_act = K_act*eye(size(Act,1));
       sigma_act_tilde_dot = [-J_a_1*zeta;zeros(8,1);sigma_actu_tilde_dot];
       sigma_act_tilde = [radius_obs+epsilon-sigma_a1;zeros(8,1);sigma_actuation_tilde];
       J_a12 = [J_a1;J_2];
       J_a12 = comp_lin_indep_cols(J_a12')';
       J_a12_inv = J_a12'/(J_a12*J_a12');
       N_a12 = eye(14) - J_a12_inv*J_a12;
       
       
        for i=1:size(theta,1)
            if theta(i) < 0
                sigma_act_tilde(i+1) = -theta_max-theta(i);
                sigma_act_tilde_dot(i+1) = -zeta(6+i);
            else
                sigma_act_tilde(i+1) = theta_max-theta(i);
                sigma_act_tilde_dot(i+1) = -zeta(6+i);
            end
        end
        
        sigma_act_tilde = Act*sigma_act_tilde;
        sigma_act_tilde_dot = Act*sigma_act_tilde_dot;
        J_a_dot = Act*J_a_dot;

        if ~isequal(tasks,tasks_prev)
            t_switch = t;
            f2_prev = f2;
            mode_prev = mode;
        end
       if ismember(1,tasks)
           Sigma_act(1,1) = K_obs;
           Lambda_act(1,1) = K_obs_dot;
           mode = 9 + size(Act,1);
       elseif ismember(10,tasks)
           mode = 19 + size(Act,1);
       else
           mode = 1 + size(Act,1);
       end
       
       f2 =  J_a_act_inv*(Sigma_act*sigma_act_tilde+Lambda_act*sigma_act_tilde_dot-J_a_dot*zeta) + N_a*J_1_inv*(Sigma_1*sigma_1_tilde + Lambda_1_dot*sigma_1_tilde_dot - J_1_dot*zeta) + N_a1*J_2_inv*(Sigma_2*sigma_2_tilde + Lambda_2_dot*sigma_2_tilde_dot - J_2_dot*zeta)+N_a12*J_3_inv*Lambda_3*sigma_3_tilde_dot;
       
       if mode_prev == 1
           zeta_d_dot = (1-smoothing_function(t,t_switch,a,b)).*f1 + smoothing_function(t,t_switch,a,b).*f2;
       else
           zeta_d_dot = (1-smoothing_function(t,t_switch,a,b)).*f2_prev + smoothing_function(t,t_switch,a,b).*f2;
       end
    end

    a_b = zeta_d_dot;
    tau = M*a_b + c + d + g;
%     tau = M*a_b + g;
    B_joint = [zeros(6,8);eye(8)];
    B_full = [B B_joint];
    lambda = 0.0;
%     W = diag([.3 .3 .3 .3 .3 .3 .3 2 2 2 2 2 2 2 2]);
    W = eye(14);
%     u = W\B_full'/(B_full/W*B_full'+lambda^2*eye(14))*tau;
%     tau_c = M(1:6,1:6)*zeta_d_dot(1:6) + g(1:6) + c(1:6) + d(1:6);
%     thrust = B(1:6,:)'/(B(1:6,:)*B(1:6,:)' + 0*eye(6))*tau_c;
    
%     u = pinv(B_full)*tau;
    u = B_full'/(B_full*B_full')*tau;
%     u = W\B_full'/(B_full*W\B_full')*tau;
    thrust = u(1:7);
    joint_torque = u(8:end);
%     joint_torque = M(7:end,7:end)*zeta_d_dot(7:end) + g(7:end) + c(7:end) + d(7:end);
%     thrust = 60*sat(thrust, 60*ones(7,1));
%     joint_torque = 50*sat(joint_torque, 50*ones(8,1));





disp(['Elapsed time: ' num2str(t)])

t_prev = t;
%Compute the relative acceleration
zeta_dot      =  M\(B*thrust+[zeros(6,1);joint_torque]-c-d-g);
% zeta_dot      =  M\(tau-c-d-g);
%Remove the effect of current
zeta_dot(1:3) = zeta_dot(1:3)-cross(nu(4:6),velc_h);
%Compute the time derivatives of position and orientation
pos_dot   =  R(:,:,1)*nu(1:3);
quat_dot  =  TransAngQ(quat)*nu(4:6)+1*(1-quat'*quat)*quat; 

% Integral state in feedback linearizing controller
% zeta_d_dot = zeta-zeta_d;
%Assemble derivative of the state vector
x_dot = [pos_dot ; quat_dot ; zeta(7:snake.n+5) ; zeta_dot;pos; zeta; thrust; joint_torque; p_endeff; eulang_endeff; sigma_1d;att_d; sigma_2d; theta; mode;zeta_d_dot; sigma_a1;J_a_1*zeta;act_index];

% nin  = nargin;
% if nin == 5 && flag ==1
%     x_dot = [pos; zeta; thrust; joint_torque; zeta_d; p_endeff; eulang_endeff; sigma_1d;att_d; sigma_2d; theta; theta_d; mode];
% end

end
