function x_dot = dyn_attitude(x)
snake = evalin('base', 'snake');
pp = evalin('base','pp');
%Get states
pos       =  x(1:3);
quat      =  x(4:7);
theta     =  x(8 : 6+snake.n);
zeta      =  x(7+snake.n : 11+2*snake.n);
theta_d   =  x(12+2*snake.n : 10+3*snake.n);
zeta_d    =  x(11+3*snake.n : 24+3*snake.n);
t         =  x(end);


%Compute rotation matrices and jacobians
[R, J]    =  comp_rot_jac(quat,theta,snake);

J_dot    = comp_jac_dot(theta, snake, J, zeta(7:end));

%Compute mass matrix
M         =  comp_mass(J,snake);

%Compute relative velocity to take into account the effect of current
velc_h    =  R(:,:,1)'*pp.velc;
nu        =  zeta(1:6);
zeta(1:3) =  zeta(1:3)-velc_h;

%Compute coriolis, drag, and hydrostatic terms with the relative velocity
[c, d, g] =  comp_cdg(J,R,theta,snake,pp,zeta);
%Compute the thrust-configuration matrix
B         =  comp_TCM(J,snake);

%testing with a swimming gait, add your own controller here:
    % Homogenous transformation matrix between body-fixed base frame and
    % end-effector frame
    H_be = snake.joint(1).A(theta(1))*snake.joint(2).A(theta(2))*snake.joint(3).A(theta(3))*snake.joint(4).A(theta(4))*snake.joint(5).A(theta(5))*snake.joint(6).A(theta(6))*snake.joint(7).A(theta(7))*snake.joint(8).A(theta(8));
    % Position of end-effector in body-fixed base frame.
    pos_endeff = H_be*[snake.link(9).length;0;0;1];
    % Position of end-effecter in the inertial frame.
%     p_endeff = pos + R(:,:,1)*pos_endeff(1:3);
    p_endeff = pos + R(:,:,1)*pos_endeff(1:3);
    
    
    eulang_endeff = rotm2eul(R(:,:,9), 'XYZ')';
    quat_endeff = rotm2quat(R(:,:,9))';
    T_0b = TransAngQ(quat_endeff);
    if t < 50
        psi_d = pi/2;
        sigma_1d = [0;2;-20];
    else
        psi_d = 0;
        sigma_1d = [3;0;-20];
    end
    psi_d_dot = 0;
    att_d = [0 0 psi_d]';
    quat_d = eul2quat(att_d', 'XYZ')';
    eta_d = quat_d(1);
    epsilon_d = quat_d(2:4);
    eta = quat_endeff(1);
    epsilon = quat_endeff(2:4);
    % Quaternion error
    qe = [eta_d*eta+epsilon_d'*epsilon;
         eta*epsilon_d-eta_d*epsilon-Smtrx(epsilon_d)*epsilon];
     
    V_e = J(:,:,9)*zeta;
    q_dot = T_0b*V_e(4:6);

    
    
%     J_1 = [R_d(:,:,9) zeros(3,3); zeros(3,3) T_0b(2:4,:)]*J_d(:,:,9);
%     J_2 = [R_d(:,:,1) zeros(3,3+snake.n-1)];      % Base Jacobian
    J_1 = [R(:,:,9) zeros(3,3); zeros(3,3) T_0b(2:4,:)]*J(:,:,9);
%     J_1 = J(:,:,9);
%     J_1_dot = J_dot;
%     J_1 = [R(:,:,9) zeros(3,3)]*J(:,:,9);
%     q_dot = T_0b*zeta(4:6);
    T_0b_dot = 1/2.*[0 -1 0 0 0 0 -1 0 0 0 0 -1;
                1 0 0 0 0 0 0 -1 0 0 1 0;
                0 0 0 1 1 0 0 0 0 -1 0 0;
                0 0 -1 0 0 1 0 0 1 0 0 0]*kron(eye(3),q_dot);
    J_1_dot = [R(:,:,9)*Smtrx(V_e(4:6)) zeros(3,3); zeros(3,3) T_0b_dot(2:4,:)]*J(:,:,9) + [R(:,:,9) zeros(3,3); zeros(3,3) T_0b(2:4,:)]*J_dot(:,:,9);
    
    J_2 = [R(:,:,1) zeros(3,3+snake.n-1)];      % Base Jacobian
    J_2_dot = [R(:,:,1)*Smtrx(zeta(4:6)) zeros(3,3+snake.n-1)];
%     J_2_dot = [Smtrx(zeta(4:6))*R(:,:,1) zeros(3,3+snake.n-1)];
    
    pos_obs = [2 2 -60]';
    radius_obs = 1.5;
    epsilon = 0.2;
    
%     sigma_a = sqrt((pos_obstacle-p_endeff_d)'*(pos_obstacle-p_endeff_d));
    sigma_a1 = sqrt((pos_obs-p_endeff)'*(pos_obs-p_endeff));
    sigma_a2 = sqrt((pos_obs-pos)'*(pos_obs-pos));
%     J_a = -(pos_obstacle-p_endeff_d)'/(sigma_a+0.0001)*[R_d(:,:,9) zeros(3,3)]*J_d(:,:,9);
    J_a_1 = -(pos_obs-p_endeff)'/(sigma_a1)*[R(:,:,9) zeros(3,3)]*J(:,:,9);
    J_a2 = -(pos_obs-pos)'/(sigma_a2)*J_2;
%     sigma_a_tilde = epsilon+radius_obs - sigma_a;
    sigma_a_min = epsilon+radius_obs;
    sigma_a_max = Inf;
    
    jac_sigmadot_p = eye(3)/sigma_a1 - (pos_obs-p_endeff)*(pos_obs-p_endeff)'/sigma_a1^3;
    J_a1_dot = (R(:,:,9)*V_e(1:3))'*jac_sigmadot_p*[R(:,:,9) zeros(3,3)]*J(:,:,9)-(pos_obs-p_endeff)'/(sigma_a1)*([R(:,:,9)*Smtrx(V_e(4:6)) zeros(3,3)]*J(:,:,9)+[R(:,:,9) zeros(3,3)]*J_dot(:,:,9));

    J_a_dot = [J_a1_dot; zeros(8,14)];


    J_b = [0 0 0 0 0 0 1 0 0 0 0 0 0 0;
           0 0 0 0 0 0 0 1 0 0 0 0 0 0;
           0 0 0 0 0 0 0 0 1 0 0 0 0 0;
           0 0 0 0 0 0 0 0 0 1 0 0 0 0;
           0 0 0 0 0 0 0 0 0 0 1 0 0 0;
           0 0 0 0 0 0 0 0 0 0 0 1 0 0;
           0 0 0 0 0 0 0 0 0 0 0 0 1 0;
           0 0 0 0 0 0 0 0 0 0 0 0 0 1];
   
    J_a = [J_a_1;J_b];
%     J_a = J_

%     K_1 = 0.1*30;         % kinda works!
%     K_2 = 0.001*30;
%     K_3 = 0.1*25;
%     K_4 = 0.001*25;
%     K_act = 0.001*80;
%     K_act_dot = 0.1*50;
    lambda1 = 0.4;
    lambda2 = 0.4;
    K_1 = 2*lambda1;
    K_2 = 2*lambda2;
    K_3 = 2*K_1;
    K_4 = 2*K_3;
    K_act = 5;
    K_act_dot = 30;
%     K_act = 10;
%     K_act_dot = 100;
%     sigma_1d = [0;2;0];                     % Desired end-effector position
%     sigma_1d = [3.37*cos(psi_d);3.37*sin(psi_d); 0];
    sigma_1dotd = [-psi_d_dot*3.37*sin(psi_d) psi_d_dot*3.37*cos(psi_d) 0 0 0 psi_d_dot]';
%     Sigma_1 = K_1*eye(6); 
%     Sigma_1(4,4)=0; Sigma_1(5,5) = 0; Sigma_1(6,6) = 0;
%     Sigma_1(1,1) = 0.25; Sigma_1(2,2) = 0.25;
%     Sigma_1(1,1) = 0.5; Sigma_1(2,2) = 0.5; Sigma_1(3,3) = 0.2;
%     sigma_1_tilde = [sigma_1d - p_endeff; -qe(2:4)];       
    Sigma_1 = K_1*eye(6);
    sigma_1_tilde = [sigma_1d - p_endeff; qe(2:4)];
%     sigma_1_tilde_dot = [-R(:,:,9)*V_e(1:3); qe_dot(2:4)];
    sigma_1_tilde_dot = [-R(:,:,9)*V_e(1:3); -R(:,:,9)*V_e(4:6)];
%     Lambda_1_dot = [K_3*eye(3) zeros(3); zeros(3) zeros(3)]; 
    Lambda_1_dot = K_3*eye(6);
    
%     sigma_2d = [0 -1 1.5]';
    sigma_2d = [0 0 -20]';
    
    Sigma_2 = K_2*eye(3);
    Lambda_2_dot = K_4*eye(3);
%     Sigma_2 = 0.1*eye(3);
    sigma_2_tilde = sigma_2d - pos;
    sigma_2_tilde_dot = - R(:,:,1)*zeta(1:3);

%     W = diag([0.3 0.3 0.3 0 0.3 0.3 0.7 0.7 1 1 1 1 1 1]);
%     W = eye(14);
%     J_1_inv = W\J_1'/(J_1/W*J_1');
    J_1_inv = J_1'/(J_1*J_1'+0.0*eye(size(J_1,1)));
    J_2_inv = J_2'/(J_2*J_2'+0.0*eye(size(J_2,1)));
%     J_1_inv = pinv(J_1);
%     J_2_inv = pinv(J_2);
    N_1 = eye(14) - J_1_inv*J_1;

    f1 = J_1_inv*(Sigma_1*sigma_1_tilde + Lambda_1_dot*sigma_1_tilde_dot - J_1_dot*zeta) + N_1*J_2_inv*(Sigma_2*sigma_2_tilde + Lambda_2_dot*sigma_2_tilde_dot - J_2_dot*zeta);
%     zeta_d_dot = f1;
    theta_max = 60*pi/180;
    sigma_b_max = theta_max.* ones(8,1);
    sigma_b_min = -sigma_b_max;
    % Extension to vector tasks
    [Act,tasks] = calc_activation_matrix_mod(J_a*f1+J_a_dot*zeta, J_a*zeta, [sigma_a1;theta], [sigma_a_min;sigma_b_min], [sigma_a_max;sigma_b_max]);
    if nnz(Act) == 0
        zeta_d_dot = f1;
        mode = 1;
    else
       if ismember(1,tasks)
           mode = 10;
       else
           mode = 1 + size(Act,1);
       end
       
       J_a_act = Act*J_a;
       J_a_act_inv = J_a_act'/(J_a_act*J_a_act');
       N_a = (eye(14) - J_a_act_inv*J_a_act);
       J_a1 = [J_a_act;J_1];
%        J_a1_inv = J_a1'/(J_a1*J_a1' + 0.000*eye(size(J_a1,1)));
       J_a1_inv = pinv(J_a1);
       N_a1 = (eye(14)-J_a1_inv*J_a1);
       Lambda_act = K_act_dot*eye(size(Act,1));
       Sigma_act = K_act*eye(size(Act,1));
       sigma_act_tilde = [sigma_a_min-sigma_a1;zeros(8,1)];
       sigma_act_tilde_dot = [-J_a_1*zeta;zeros(8,1)];
       sigma_act_d_ddot = [0;zeros(8,1)];
       for i=1:size(theta,1)
%            sigma_act_tilde_dot(i) = -zeta(6+i);
           if theta(i) < 0
                sigma_act_tilde(i+1) = -theta_max-theta(i);
                sigma_act_tilde_dot(i+1) = -zeta(6+i);
%                 sigma_act_d_ddot(i+1) = 0.1;
            else
                sigma_act_tilde(i+1) = theta_max-theta(i);
                sigma_act_tilde_dot(i+1) = -zeta(6+i);
%                 sigma_act_d_ddot(i+1) = -0.1;
            end
       end
       j = 1;
       for i=1:size(sigma_act_tilde_dot,1)
            if ~ismember(i,tasks)
                J_a_dot(j,:) = [];
                sigma_act_tilde_dot(j) = [];
                sigma_act_tilde(j) = [];
                sigma_act_d_ddot(j) = [];
            else
                j = j + 1;
            end
        end
       zeta_d_dot = J_a_act_inv*(sigma_act_d_ddot + Sigma_act*sigma_act_tilde+Lambda_act*sigma_act_tilde_dot - J_a_dot*zeta) + N_a*J_1_inv*(Sigma_1*sigma_1_tilde + Lambda_1_dot*sigma_1_tilde_dot - J_1_dot*zeta) + N_a1*J_2_inv*(Sigma_2*sigma_2_tilde + Lambda_2_dot*sigma_2_tilde_dot - J_2_dot*zeta);
%         zeta_d_dot = N_a*J_1_inv*(Sigma_1*sigma_1_tilde + Lambda_1_dot*sigma_1_tilde_dot - J_1_dot*zeta);
    if (size(J_a_act,1) ~= rank(J_a_act))
           size(J_a_act,1)
           rank(J_a_act)
           disp('NOT FULL RANK!')
           disp('NOT FULL RANK!')
           disp('NOT FULL RANK!')
           disp('NOT FULL RANK!')
    end
    end


%     zeta_d_dot = (zeta_d-zeta_d_prev)/0.001;
%     a_b =  zeta_d_dot;
%     a_b = -Kd*(zeta-zeta_d)-Kp*e;
%     a_b = J_2_inv*(Sigma_2*sigma_2_tilde + Lambda_2_dot*sigma_2_tilde_dot - J_2_dot*zeta);
    a_b = zeta_d_dot;
%     tau = M*a_b + c + d + g;
    tau = M*a_b;
    B_joint = [zeros(6,8);eye(8)];
    B_full = [B B_joint];
    lambda = 0.001;
%     W = diag([2 2 2 2 2 2 2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3]);
    W = eye(15);
    u = W\B_full'/(B_full/W*B_full'+lambda^2*eye(14))*tau;
    thrust = u(1:7);
    joint_torque = u(8:end);

%     thrust = 40*sat(thrust, 40*ones(7,1));
%     joint_torque = 50*sat(joint_torque,50*ones(8,1));
%     joint_torque = - Kp*(theta-theta_d)- Kd*(zeta(7:snake.n+5)-zeta_d(7:snake.n+5)); %PD-controller
%     thrust = B(1:6,:)'/(B(1:6,:)*B(1:6,:)'+lambda*eye(6))*tau(1:6);
%     thrust = 50*sat(thrust, 50*ones(1,7));
%     joint_torque = M(7:end,7:end)*a_b(7:end);
%     joint_torque = 60*sat(joint_torque, 60*ones(1,8));
%     joint_torque = M(7:end,7:end)*(zeta_dotd(7:end)-Kp*(zeta(7:end)-zeta_d(7:end)) - Ki*(theta-theta_d)) + c(7:end) + d(7:end) + g(7:end);
%     joint_torque = zeros(snake.n - 1, 1);

disp(['Elapsed time: ' num2str(t)])

% P21 = J_2*J_1_inv;
% svd(P21)
% P22 = J_2*(eye(14)-J_1_inv*J_1)*J_2_inv;
% svd(P22)

%Compute the relative acceleration
zeta_dot      =  M\(B*thrust+[zeros(6,1);joint_torque]-c-d-g);
%Remove the effect of current
zeta_dot(1:3) = zeta_dot(1:3)-cross(nu(4:6),velc_h);
%Compute the time derivatives of position and orientation
pos_dot   =  R(:,:,1)*nu(1:3); 
quat_dot  =  TransAngQ(quat)*nu(4:6)+1*(1-quat'*quat)*quat; 

% Integral state in feedback linearizing controller
% zeta_d_dot = zeta-zeta_d;
%Assemble derivative of the state vector
x_dot = [pos_dot ; quat_dot ; zeta(7:snake.n+5) ; zeta_dot; zeta_d(7:snake.n+5); zeta_d_dot;pos; zeta; thrust; joint_torque; p_endeff; eulang_endeff; sigma_1d;att_d; sigma_2d; theta; mode;zeta_d_dot; sigma_a1;J_a_1*zeta];

% nin  = nargin;
% if nin == 5 && flag ==1
%     x_dot = [pos; zeta; thrust; joint_torque; zeta_d; p_endeff; eulang_endeff; sigma_1d;att_d; sigma_2d; theta; theta_d; mode];
% end

end
