close all

figure
hold on;
plot(simout.time, simout.signals.values(:,25:25+7)', 'LineWidth',1.5)
xlabel('Time [s]','interpreter','latex');
ylabel('Joint torque [Nm]','interpreter','latex')
legend({'$u_8$', '$u_9$','$u_{10}$','$u_{11}$', '$u_{12}$','$u_{13}$', '$u_{14}$', '$u_{15}$'},'Interpreter','latex','Location','best','NumColumns',2)
% title('Control torque')
grid on
print -depsc2 -painters joint-torque.eps

figure
plot(simout.time, simout.signals.values(:,18:24)', 'LineWidth',1.5)
xlabel('Time [s]','interpreter','latex');
ylabel('Thrust [N]','interpreter','latex')
legend({'$u_1$', '$u_2$','$u_3$','$u_4$', '$u_5$','$u_6$', '$u_7$'},'Interpreter','latex','Location','best','NumColumns',2)
% title('Thruster force')
grid on
print -depsc2 -painters thrust.eps




figure
th = 0:pi/50:2*pi;
r = .3;
a = 2.5;
b = 0.5;
c = -10;
xunit = r * cos(th) + 0.5;
yunit = r * sin(th) + 2.5;

hold on
plot(simout.signals.values(:,2)', simout.signals.values(:,1)')
plot(simout.signals.values(:,34)', simout.signals.values(:,33)')
plot(simout.signals.values(:,40)', simout.signals.values(:,39)')
plot(xunit, yunit);
legend({'$p_b$', '$p_e$', '$p_d$'},'Interpreter','latex')
title('North-West plot')
set(gca, 'XDir','reverse')
axis('equal')


figure
plot3(simout.signals.values(:,33)', simout.signals.values(:,34)', simout.signals.values(:,35)', 'm', 'linewidth', 1.5);
hold on
% plot3(xunit, yunit, -10*ones(size(xunit,2)))
plot3(simout.signals.values(:,1)', simout.signals.values(:,2)', simout.signals.values(:,3)', 'k', 'linewidth', 1);
plot3(simout.signals.values(:,39)', simout.signals.values(:,40)', simout.signals.values(:,41)','r','linewidth', 1.5)
[x,y,z] = sphere;
surf(x*r+a, y*r+b, z*r+c);
zlim([-11 -9])
legend({'$p_e$', '$p_b$','$p_d$'},'Interpreter','latex')
% legend({'$p_e$','$p_d$'},'Interpreter','latex')
xlabel('North [m]','interpreter','latex');
ylabel('West [m]','interpreter','latex');
zlabel('Up [m]','interpreter','latex');
axis('equal')
grid on
print -depsc2 -painters north-west.eps


figure
hold on;
grid on
plot(simout.time, 180/pi.*simout.signals.values(:,48:55)', 'LineWidth',1)
plot([simout.time(1) simout.time(end)], [60 60], '--', 'LineWidth',1)
plot([simout.time(1) simout.time(end)], [-60 -60], '--', 'LineWidth',1)
xlabel('Time [s]','interpreter','latex');
ylabel('Angle [deg]','interpreter','latex')
ylim([-70 70])
legend({'${\theta}_1$', '${\theta}_2$','${\theta}_3$','${\theta}_4$','${\theta}_5$','${\theta}_6$','${\theta}_7$','${\theta}_8$'},'Interpreter','latex','Location','Best','NumColumns',4)
print -depsc2 -painters joint-angles.eps



figure
subplot(3,1,1)

hold on;
grid on
plot(simout.time, simout.signals.values(:,1)', '-g', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,45)','--')
plot(simout.time, simout.signals.values(:,33)', 'r', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,39)','--')
legend({'$x_b$', '$x_{b,d}$', '$x_e$', '$x_{e,d}$'},'Interpreter','latex','Location','northwest','NumColumns',2)

ylabel('Position [m]','interpreter','latex')
xlabel('Time [s]','interpreter','latex');
subplot(3,1,2)
hold on;
grid on
plot(simout.time, simout.signals.values(:,2)', '-g', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,46)', '--')
plot(simout.time, simout.signals.values(:,34)', 'r', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,40)', '--')
legend({'$y_b$', '$y_{b,d}$','$y_e$','$y_{e,d}$'},'Interpreter','latex','Location','west','NumColumns',2)

ylabel('Position [m]','interpreter','latex')
xlabel('Time [s]','interpreter','latex');
subplot(3,1,3)

hold on;
plot(simout.time, simout.signals.values(:,3)', '-g', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,47)', '--' )
plot(simout.time, simout.signals.values(:,35)', 'r', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,41)', '--')
legend({'$z_b$', '$z_{b,d}$','$z_e$','$z_{e,d}$'},'Interpreter','latex','Location','northwest','NumColumns',2)
ylim([-11 -9])
xlabel('Time [s]','interpreter','latex');
ylabel('Position [m]','interpreter','latex')
grid on
print -depsc2 -painters pos.eps


figure
hold on;
plot(simout.time, 180/pi.*simout.signals.values(:,36)', 'LineWidth',1.5)
plot(simout.time, 180/pi.*simout.signals.values(:,37)', 'LineWidth',1.5)
plot(simout.time, 180/pi.*simout.signals.values(:,38)', 'LineWidth',1.5)
plot(simout.time, refmod.signal3.data(:,8:10).*180/pi,'--')
% plot(simout.time, refmod.signal2.data(:,8:10).*180/pi)
legend({'$\phi$','$\theta$','$\psi$','$\phi_d$','$\theta_d$', '$\psi_d$'},'interpreter','latex')
xlabel('$t$ [s]','interpreter','latex');
ylabel('Angle [deg]','interpreter','latex')
grid on
print -depsc2 -painters attitude.eps


figure
hold on
plot(simout.time, simout.signals.values(:,71), 'LineWidth',1.5)
plot([simout.time(1) simout.time(end)], [.32 .32], '--', 'LineWidth',1)
% title('Distance between end-effector and obstacle')
xlabel('Time [s]','interpreter','latex');
ylabel('Distance [m]','interpreter','latex')
legend({'$\sigma_{a}$','$r_{obs}+\epsilon$'},'Interpreter','latex')
grid on
print -depsc2 -painters dist-obs.eps

figure
hold on
plot(simout.time, simout.signals.values(:,73),'LineWidth', 1.5)
plot([simout.time(1) simout.time(end)], [0.1 0.1], '--', 'LineWidth',1)
xlabel('Time [s]','interpreter','latex');
ylabel('Actuation index','interpreter','latex')
legend({'$\sigma_{b}$', '$\sigma_{b,min}$'},'Interpreter','latex')
grid on
print -depsc2 -painters act-index.eps