%This is a simple smiulator for an underwater snake-like robot. The code is
%not optimized for speed and there may still be bugs. Please report back to
%me if you find anything fishy :)
%
%Henrik

clc;
clear
close all

C_1 = eye(29,102);
C_2 = [zeros(73,29) eye(73,73)];
lambda1 = 0.2;
lambda2 = 0.35;
lambda3 = 0.1;
K_1 = lambda1^2;
K_att = lambda2^2;
K_3 = 2*lambda1;
K_att_dot = 2*lambda2;
K_2 = lambda3^2;
K_4 = 2*lambda3;




% C_1 = eye(58,137);
% C_2 = [zeros(79,58) eye(79,79)];

%% Define Physical Parameters
pp.denw = 1000;         %density of water
pp.grav = 9.81;         %gravitational acceleration
pp.velc = [0 0 0]';     %constant irrotational current velocity, inertial frame
pp.gdir = [0 0 -1]';    %direction of gravity in inertial frame (pointing towards ground, unit vector

%% Define Snake Parameters
% snake.link_vec  = [8 8 8 8 8];         %The links the snake consists of, from head to tail
% snake.joint_vec = [3 3 3 3];           %The type of joints, from head to tail
snake.link_vec  = [1 2 3 2 4 2 5 2 6];         
snake.joint_vec = [3 2 3 2 3 2 3 2];          
snake.n         = length(snake.link_vec);

C_a_C   =   1; % Added mass coefficient for the cross section, 1 is the theoretical inviscid result
C_d_1   =  .2; % nonlinear drag coefficient in surge
C_d_4   =  .1; % nonlinear drag coefficient in the roll direction for the cross-section
C_d_C   =  .5; % nonlinear crossflow drag coefficient
C_d_L   =  .1; % Linear cross-sectional drag coefficient
alpha   =  .2; % Added mass ratio in surge/heave for a link
beta    =  .1; % Linear drag parameter in surge
gamma   =  .1; % Linear drag parameter in roll

masscoeff = [C_a_C, alpha];
dragcoeff = [C_d_1, C_d_4, C_d_C, C_d_L, beta, gamma];
clear C_a_C C_d_C C_d_1 C_d_4 C_d_L alpha beta gamma

%% Define Link Properties
link = define_link_properties_eely(pp, masscoeff, dragcoeff);

%% Define Joint Properties
joint = define_joint_properties();
                  
%% Define Simulation Parameters
Tmax          = 400;
deltaT_interp = 1;                     %Time step size for the interpolated solution
deltaT_plot   = .1;                      %Time step size for the plotted solution

int_0           = zeros(14,1);
y_0             = zeros(29,1); %Initial conditions
y_0(3)          = -10;
y_0(4)          = 1;                     %"zero" quaternion

y_0(8:snake.n+6) = [pi/6 0 pi/6 0 pi/6 0 pi/6 0]'; % initialize with
%U-shape

                  
%% Build Snake
snake = build_snake(snake, link, joint, pp);
% Simulink.data.evalinGlobal('USMsim','snake = build_snake(snake, link, joint, pp);')

%% Simulate
disp('Simulation starting:')
disp('...')
% [T, simout]   = ode45(@(t,x)dyn(t,x,snake,pp,0),[0 Tmax], y_0);
options = simset('SrcWorkspace','current');
sim('run_smooth.slx',Tmax,options)
disp('Simulation done!')

%% Post Process/Show Movie
% this code shows the correct result if all links have the same radius!
% disp('Postprocessing data:')
% disp('...')
% Tinterp      = 0 : deltaT_interp : Tmax;
% simout_int   = interp1(T,simout,Tinterp);
% TF = ismissing(simout_int);
% k = find(TF,1);
% if isempty(k), k = length(Tinterp); end
% simout_int = simout_int(1:k-1,:);
% [X, Y, Z]    = snake2points(simout_int,snake);
% mov          = animatesnake(X,Y,Z,snake);

% Tinterp      = 0 : deltaT_plot : Tmax;
% simout_int   = interp1(T,simout,Tinterp);
% TF = ismissing(simout_int);
% k = find(TF,1);
% if isempty(k), k = length(Tinterp); end
% simout_int = simout_int(1:k-1,:);
% simout = dyn(Tinterp(1),simout_int(1,:)',snake,pp,1);
% simout = [simout, zeros(length(simout),k-2)];
% for t = 2:k-1
%     simout(:,t) = dyn(Tinterp(t),simout_int(t,:)',snake,pp,1);
% end
% save('log_moe.mat', 'simout')
% disp('Postprocessing done.')
%%
% [X, Y, Z]    = snake2points(simout.signals.values,snake);
% mov          = animatesnake(X,Y,Z,snake);


figure
hold on;
plot(simout.time, simout.signals.values(:,25:25+7)', 'LineWidth',1)
xlabel('Time [s]','interpreter','latex');
ylabel('Joint torque [Nm]','interpreter','latex')
legend({'$u_8$', '$u_9$','$u_{10}$','$u_{11}$', '$u_{12}$','$u_{13}$', '$u_{14}$', '$u_{15}$'},'Interpreter','latex','Location','best','NumColumns',2)
% title('Control torque')
grid on
print -depsc2 -painters joint-torque.eps

figure
plot(simout.time, simout.signals.values(:,18:24)', 'LineWidth',1)
xlabel('Time [s]','interpreter','latex');
ylabel('Thrust [N]','interpreter','latex')
legend({'$u_1$', '$u_2$','$u_3$','$u_4$', '$u_5$','$u_6$', '$u_7$'},'Interpreter','latex','Location','best','NumColumns',2)
% title('Thruster force')
grid on
print -depsc2 -painters thrust.eps


figure
subplot(2,1,1)

hold on
% plot(simout.time, simout.signals.values(:,57:70));
plot(simout.time, simout.signals.values(:,57:59), 'LineWidth',1)
xlabel('Time [s]','interpreter','latex');
ylabel('Linear acceleration [$m/s^2$]','interpreter','latex')
legend({'$\dot{\zeta}_1$', '$\dot{\zeta}_2$','$\dot{\zeta}_3$'},'Interpreter','latex','Location','best','NumColumns',2)


subplot(2,1,2)
plot(simout.time, simout.signals.values(:,60:70), 'LineWidth',1)
ylabel('Angular acceleration [$rad/s^2$]','interpreter','latex')
xlabel('Time [s]','interpreter','latex');
legend({'$\dot{\zeta}_4$','$\dot{\zeta}_5$','$\dot{\zeta}_6$', '$\dot{\zeta}_7$','$\dot{\zeta}_8$', '$\dot{\zeta}_9$', '$\dot{\zeta}_{10}$', '$\dot{\zeta}_{11}$', '$\dot{\zeta}_{12}$', '$\dot{\zeta}_{13}$','$\dot{\zeta}_{14}$'},'Interpreter','latex','Location','best','NumColumns',6)
print -depsc2 -painters references.eps




%% Custom plotting
% figure
% hold on
% plot(simout.time, simout.signals.values(:,2))
% plot(simout.time, simout(33,:)', '--')
% title('Surge velocity')     



figure
th = 0:pi/50:2*pi;
r = .3;
a = 2.5;
b = 0.5;
c = -10;
xunit = r * cos(th) + 0.5;
yunit = r * sin(th) + 2.5;

hold on
plot(simout.signals.values(:,2)', simout.signals.values(:,1)')
plot(simout.signals.values(:,34)', simout.signals.values(:,33)')
scatter(simout.signals.values(:,40)', simout.signals.values(:,39)')
plot(xunit, yunit);
legend({'$p_b$', '$p_e$', '$p_d$'},'Interpreter','latex')
title('North-West plot')
set(gca, 'XDir','reverse')
axis('equal')

figure
plot3(simout.signals.values(:,33)', simout.signals.values(:,34)', simout.signals.values(:,35)', 'k', 'linewidth', 1);
hold on
% plot3(xunit, yunit, -10*ones(size(xunit,2)))
plot3(simout.signals.values(:,1)', simout.signals.values(:,2)', simout.signals.values(:,3)', 'c', 'linewidth', 1);
scatter3(simout.signals.values(:,39)', simout.signals.values(:,40)', simout.signals.values(:,41)', 'filled', 'MarkerFaceColor','red')
[x,y,z] = sphere;
surf(x*r+a, y*r+b, z*r+c);
zlim([-11 -9])
legend({'$p_e$', '$p_b$','$p_d$'},'Interpreter','latex')
xlabel('North [m]','interpreter','latex');
ylabel('West [m]','interpreter','latex');
zlabel('Up [m]','interpreter','latex');
axis('equal')
print -depsc2 -painters north-west.eps

% figure
% hold on
% plot3(simout(2,:)', simout(1,:)', simout(3,:)')
% plot3(simout(48,:)', simout(47,:)', simout(49,:)')
% legend({'$p_b$', '$p_e$'},'Interpreter','latex')
% title('North-East-Down plot')
%%


figure
hold on;
plot(simout.time, 180/pi.*simout.signals.values(:,48:55)', 'LineWidth',1)
plot([simout.time(1) simout.time(end)], [60 60], '--', 'LineWidth',1)
plot([simout.time(1) simout.time(end)], [-60 -60], '--', 'LineWidth',1)
xlabel('Time [s]','interpreter','latex');
ylabel('Angle [deg]','interpreter','latex')
ylim([-70 70])
legend({'${\theta}_1$', '${\theta}_2$','${\theta}_3$','${\theta}_4$','${\theta}_5$','${\theta}_6$','${\theta}_7$','${\theta}_8$'},'Interpreter','latex','Location','Best','NumColumns',4)
print -depsc2 -painters joint-angles.eps

figure
plot(simout.time, simout.signals.values(:,56), 'LineWidth',1)
xlabel('Time [s]','interpreter','latex');
ylabel('Mode','interpreter','latex')
grid on
print -depsc2 -painters mode.eps
%%

%%

figure
subplot(3,1,1)

hold on;
plot(simout.time, simout.signals.values(:,1)', '-g', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,45)','--', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,33)', 'r', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,39)','--', 'LineWidth',1)
legend({'$x_b$', '$x_{b,d}$', '$x_e$', '$x_{e,d}$'},'Interpreter','latex','Location','northwest','NumColumns',2)
grid on
ylabel('Position [m]','interpreter','latex')
xlabel('Time [s]','interpreter','latex');

subplot(3,1,2)
hold on;
plot(simout.time, simout.signals.values(:,2)', '-g', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,46)', '--', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,34)', 'r', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,40)', '--', 'LineWidth',1)
legend({'$y_b$', '$y_{b,d}$','$y_e$','$y_{e,d}$'},'Interpreter','latex','Location','west','NumColumns',2)
grid on
ylabel('Position [m]','interpreter','latex')
xlabel('Time [s]','interpreter','latex');
subplot(3,1,3)

hold on;
plot(simout.time, simout.signals.values(:,3)', '-g', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,47)', '--', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,35)', 'r', 'LineWidth',1)
plot(simout.time, simout.signals.values(:,41)', '--', 'LineWidth',1)
legend({'$z_b$', '$z_{b,d}$','$z_e$','$z_{e,d}$'},'Interpreter','latex','Location','northwest','NumColumns',2)
ylim([-11 -9])
xlabel('Time [s]','interpreter','latex');
ylabel('Position [m]','interpreter','latex')
grid on
print -depsc2 -painters pos.eps

figure
hold on;
plot(simout.time, 180/pi.*simout.signals.values(:,36)', 'LineWidth',1)
plot(simout.time, 180/pi.*simout.signals.values(:,37)', 'LineWidth',1)
plot(simout.time, 180/pi.*simout.signals.values(:,38)', 'LineWidth',1)
plot(simout.time, 180/pi.*simout.signals.values(:,42), '--', 'LineWidth',1)
plot(simout.time, 180/pi.*simout.signals.values(:,43), '--', 'LineWidth',1)
plot(simout.time, 180/pi.*simout.signals.values(:,44), '--', 'LineWidth',1)
legend({'$\phi$','$\theta$','$\psi$','$\phi_d$','$\theta_d$', '$\psi_d$'},'interpreter','latex')
xlabel('$t$ [s]','interpreter','latex');
ylabel('Angle [deg]','interpreter','latex')
grid on
print -depsc2 -painters attitude.eps



figure
hold on
plot(simout.time, simout.signals.values(:,71), 'LineWidth',1)
plot([simout.time(1) simout.time(end)], [.3 .3], '--', 'LineWidth',1)
plot([simout.time(1) simout.time(end)], [.35 .35], '--', 'LineWidth',1)
% title('Distance between end-effector and obstacle')
xlabel('Time [s]','interpreter','latex');
ylabel('Distance [m]','interpreter','latex')
legend({'$\sigma_{a_1}$','$r_{obs}$' ,'$\sigma_{a_{1},min}$'},'Interpreter','latex')
grid on
print -depsc2 -painters dist-obs.eps

figure
hold on
plot(simout.time, simout.signals.values(:,72));
title('Change in distance between end-effector and obstacle')

figure
hold on
plot(simout.time, simout.signals.values(:,73),'LineWidth', 1)
plot([simout.time(1) simout.time(end)], [0.1 0.1], '--', 'LineWidth',1)
xlabel('Time [s]','interpreter','latex');
ylabel('Actuation index','interpreter','latex')
legend({'$\sigma_{a_3}$', '$\sigma_{a_{3},min}$'},'Interpreter','latex')
grid on
print -depsc2 -painters act-index.eps
